@@ -1,4 +1,4 @@
-/* $Id: hparser.c,v 2.32 2000/01/15 15:34:17 gisle Exp $
+/* $Id: hparser.c,v 2.38 2000/01/21 23:56:09 gisle Exp $
  *
  * Copyright 1999, Gisle Aas
  * Copyright 1999 Michael A. Chase
@@ -67,6 +67,8 @@ char *argname[] = {
 };
 
 
+static void flush_pending_text(PSTATE* p_state, SV* self);
+
 /*
  * Parser functions.
  *
@@ -95,7 +97,7 @@ report_event(PSTATE* p_state,
 	     SV* self
 	    )
 {
-  struct p_handler *h = &p_state->handlers[event];
+  struct p_handler *h;
   dSP;
   AV *array;
   STRLEN my_na;
@@ -140,6 +142,30 @@ report_event(PSTATE* p_state,
     return;
 #endif
 
+  if (p_state->unbroken_text && event == E_TEXT) {
+    /* should buffer text */
+    if (!p_state->pend_text)
+      p_state->pend_text = newSV(256);
+    if (SvOK(p_state->pend_text)) {
+      if (p_state->is_cdata != p_state->pend_text_is_cdata) {
+	flush_pending_text(p_state, self);
+	goto INIT_PEND_TEXT;
+      }
+    }
+    else {
+    INIT_PEND_TEXT:
+      p_state->pend_text_offset = p_state->chunk_offset + offset;
+      p_state->pend_text_is_cdata = p_state->is_cdata;
+      sv_setpvn(p_state->pend_text, "", 0);
+    }
+    sv_catpvn(p_state->pend_text, beg, end - beg);
+    return;
+  }
+  else if (p_state->pend_text && SvOK(p_state->pend_text)) {
+    flush_pending_text(p_state, self);
+  }
+
+  h = &p_state->handlers[event];
   if (!h->cb) {
     /* event = E_DEFAULT; */
     h = &p_state->handlers[E_DEFAULT];
@@ -286,14 +312,14 @@ report_event(PSTATE* p_state,
     case ARG_DTEXT:
       if (event == E_TEXT) {
 	arg = sv_2mortal(newSVpvn(beg, end - beg));
-	if (!CDATA_MODE(p_state))
+	if (!p_state->is_cdata)
 	  decode_entities(arg, entity2char);
       }
       break;
       
     case ARG_IS_CDATA:
       if (event == E_TEXT) {
-	arg = boolSV(CDATA_MODE(p_state));
+	arg = boolSV(p_state->is_cdata);
       }
       break;
 
@@ -436,6 +462,34 @@ argspec_compile(SV* src)
 }
 
 
+static void
+flush_pending_text(PSTATE* p_state, SV* self)
+{
+  bool   old_unbroken_text = p_state->unbroken_text;
+  SV*    old_pend_text     = p_state->pend_text;
+  bool   old_is_cdata      = p_state->is_cdata;
+  STRLEN old_chunk_offset  = p_state->chunk_offset;
+
+  assert(p_state->pend_text && SvOK(p_state->pend_text));
+
+  p_state->unbroken_text = 0;
+  p_state->pend_text     = 0;
+  p_state->is_cdata      = p_state->pend_text_is_cdata;
+  p_state->chunk_offset  = p_state->pend_text_offset;
+
+  report_event(p_state, E_TEXT,
+	       SvPVX(old_pend_text), SvEND(old_pend_text),
+	       0, 0, 0,
+	       self);
+  SvOK_off(old_pend_text);
+
+  p_state->unbroken_text = old_unbroken_text;
+  p_state->pend_text     = old_pend_text;
+  p_state->is_cdata      = old_is_cdata;
+  p_state->chunk_offset  = old_chunk_offset;
+}
+
+
 static char*
 parse_comment(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 {
@@ -573,6 +627,7 @@ marked_section_update(PSTATE* p_state)
     }
   }
   /* printf("MS %d\n", p_state->ms); */
+  p_state->is_cdata = (p_state->ms == MS_CDATA);
   return;
 }
 
@@ -900,7 +955,7 @@ parse_start(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 		   offset + (s - beg), self);
     FREE_TOKENS;
 
-    {
+    if (!p_state->xml_mode) {
       /* find out if this start tag should put us into literal_mode
        */
       int i;
@@ -920,6 +975,7 @@ parse_start(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 	    if (!--len) {
 	      /* found it */
 	      p_state->literal_mode = literal_mode_elem[i].str;
+	      p_state->is_cdata = 1;
 	      /* printf("Found %s\n", p_state->literal_mode); */
 	      goto END_OF_LITERAL_SEARCH;
 	    }
@@ -1044,6 +1100,8 @@ parse(PSTATE* p_state,
       SvREFCNT_dec(p_state->buf);
       p_state->buf = 0;
     }
+    if (p_state->pend_text && SvOK(p_state->pend_text))
+      flush_pending_text(p_state, self);
     return;
   }
 
@@ -1107,6 +1165,7 @@ parse(PSTATE* p_state,
 	    report_event(p_state, E_END,  end_text, s, &end_token, 1,
 			 end_text - beg, self);
 	    p_state->literal_mode = 0;
+	    p_state->is_cdata = 0;
 	    t = s;
 	  }
 	}
@@ -1183,7 +1242,7 @@ parse(PSTATE* p_state,
 	}
 	s++;
 	if (s != t)
-	  report_event(p_state, E_TEXT, t, s, 0, 0, beg - t, self);
+	  report_event(p_state, E_TEXT, t, s, 0, 0, t - beg, self);
 	break;
       }
     }
